@@ -47,7 +47,7 @@
 
 function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo)
 
-% Copyright (C) 2009,2010,2011 Philip Nienhuis <prnienhuis at users.sf.net>
+% Copyright (C) 2009,2010,2011,2012,2013 Philip Nienhuis <prnienhuis at users.sf.net>
 %
 % This program is free software; you can redistribute it and/or modify it under
 % the terms of the GNU General Public License as published by the Free Software
@@ -84,6 +84,13 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
 % 2011-09-18 Fixed 'Matlab style short circuit' warning in L. 152
 % 2012-12-24 Amended code stanze to find unoil.jar; now works in LibreOffice 3.5b2 as well
 % 2012-06-07 Replaced all tabs by double space
+% 2012-06-24 Replaced error msg by printf & return
+%     ''     Added Java pkg inquiry (Octave) before attempting javaclasspath()
+%     ''     Updated check for odfdom version (now supports 0.8.8)
+% 2012-10-07 Moved common classpath entry code to private function
+% 2012-12-21 POI 3.9 support (w. either xmlbeans.jar or xbeans.jar)
+% 2013-01-16 Updated to Octave w built-in Java (3.7.1+)
+% 2013-01-20 Made JVM memory detector more robust wrt Java return type
 
   jcp = []; retval = 0;
   if (nargin < 3); path_to_ooo= ''; end %if
@@ -106,33 +113,45 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
   catch
     % COM not supported
     if (dbug), fprintf ('not working.\n\n'); end %if
-    end %try_catch
+  end %try_catch
 
     % Check Java
   if (dbug), fprintf ('Checking Java support...\n'); end %if
   if (dbug > 1), fprintf ('  1. Checking Java JRE presence.... '); end %if
   % Try if Java is installed at all
   if (isOctave)
-    if (ispc)
+    oct_vsn = str2double (strsplit (OCTAVE_VERSION, '.'){1}) + ...
+              0.1 * str2double (strsplit (OCTAVE_VERSION, '.'){2});
+    if (ispc)str2double (strsplit (OCTAVE_VERSION, '.'){2});
       jtst = (system ('java -version 2> nul'));
     else
       jtst = (system ('java -version 2> /dev/null'));
     end %if
   else
     tst1 = version ('-java');
-        jtst = isempty (strfind (tst1, 'Java'));
+    jtst = isempty (strfind (tst1, 'Java'));
   end %if
   if (jtst)
-    error ('Apparently no Java JRE installed.');
+    printf ('Apparently no Java JRE installed.\n');
+    return;
   else
     if (dbug > 1), fprintf ('OK, found one.\n'); end %if
   end %if
   if (dbug > 1 && isOctave), fprintf ('  2. Checking Octave Java support... '); end %if
   try
-    jcp = javaclasspath ('-all');            % For Octave java pkg > 1.2.7
-    if (isempty (jcp)), jcp = javaclasspath; end %if  % For Octave java pkg < 1.2.8
+    % The following stanza is meant for older Octave w/o built-in Java
+    if (isOctave && oct_vsn < 3.7)
+      % Check Java package
+      [~, b] = pkg ('describe', 'java');
+      if    (strcmpi (b{:}, 'Not loaded'))
+        if (dbug > 1); printf ('Java package not loaded. First do: "pkg load java"\n'); end %if
+      elseif strcmpi (b{:}, 'Not installed')
+        if (dbug > 1); printf ('Java package is not installed.\n'); end %if
+      endif
+    end %if
+    jcp = javaclasspath ('-all');
     % If we get here, at least Java works.
-    if (dbug > 1 && isOctave), fprintf ('Java package seems to work OK.\n'); end %if
+    if (dbug > 1 && isOctave), fprintf ('Java seems to work OK.\n'); end %if
     % Now check for proper version (> 1.6.x.x)
     jver = char (javaMethod ('getProperty', 'java.lang.System', 'java.version'));
     cjver = strsplit (jver, '.');
@@ -152,12 +171,13 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
     % Under *nix the classpath must first be split up.
     % Matlab is braindead here. For ML we need a replacement for Octave's builtin strsplit()
     % This is found on ML Central (BSD license so this is allowed) & adapted for input arg order
-     if (isunix && ~iscell (jcp)); jcp = strsplit (char (jcp), ':'); end %if
+    if (isunix && ~iscell (jcp)); jcp = strsplit (char (jcp), ':'); end %if
     if (dbug > 1)
       % Check JVM virtual memory settings
       jrt = javaMethod ('getRuntime', 'java.lang.Runtime');
-      jmem = jrt.maxMemory ();
-            if (isOctave), jmem = jmem.doubleValue(); end %if
+      jmem = jrt.maxMemory ()
+	  % Some Java versions return jmem as octave_value => convert to double
+	  if (! isnumeric (jmem)); jmem = jmem.doubleValue(); end %if
       jmem = int16 (jmem/1024/1024);
       fprintf ('  Maximum JVM memory: %5d MiB; ', jmem);
       if (jmem < 400)
@@ -173,33 +193,20 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
       end %if
     end %if
     if (dbug), fprintf ('Java support OK\n'); end %if
-    catch
-    error ('No Java support found: %s.', lasterr);
+  catch
+    printf ('No Java support found.\n');
+    return
   end %try_catch
 
   if (dbug), fprintf ('\nChecking javaclasspath for .jar class libraries needed for spreadsheet I/O...:\n'); end %if
 
   % Try Java & Apache POI. First Check basic .xls (BIFF8) support
   if (dbug > 1), fprintf ('\nBasic POI (.xls) <poi-3> <poi-ooxml>:\n'); end %if
-  jpchk1 = 0; entries1 = {'poi-3', 'poi-ooxml-3'}; missing1 = zeros (1, numel (entries1));
+  entries1 = {'poi-3', 'poi-ooxml-3'}; missing1 = zeros (1, numel (entries1));
   % Only under *nix we might use brute force: e.g., strfind (javaclasspath, classname)
   % as javaclasspath is one long string. Under Windows however classpath is a cell array
   % so we need the following more subtle, platform-independent approach:
-  for jj=1:length (entries1)
-    found = 0;
-    for ii=1:length (jcp)
-      jcplst = strsplit (jcp{ii}, filesep);
-      jcpentry = jcplst {end};
-      if (~isempty (strfind (lower (jcpentry), lower (entries1{jj}))))
-        jpchk1 = jpchk1 + 1; found = 1;
-        if (dbug > 2), fprintf ('  - %s OK\n', jcp{ii}); end %if
-      end %if
-    end %for
-    if (~found)
-      if (dbug > 2), fprintf ('  %s....jar missing\n', entries1{jj}); end %if 
-      missing1(jj) = 1; 
-    end %if
-  end %for
+  [jpchk1, missing1] = chk_jar_entries (jcp, entries1, dbug);
   if (jpchk1 >= numel (entries1)), retval = retval + 2; end %if
   if (dbug > 1)
     if (jpchk1 >= numel (entries1))
@@ -209,25 +216,10 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
     end %if
   end %if
   % Next, check OOXML support
-    if (dbug > 1), fprintf ('\nPOI OOXML (.xlsx) <xbean> <poi-ooxml-schemas> <dom4j>:\n'); end %if
-  jpchk2 = 0; entries2 = {'xbean', 'poi-ooxml-schemas', 'dom4j-1.6.1'}; 
-  missing2 = zeros (1, numel (entries2));
-  for jj=1:length (entries2)
-    found = 0;
-    for ii=1:length (jcp)
-      jcplst = strsplit (jcp{ii}, filesep);
-      jcpentry = jcplst {end};
-      if (~isempty (strfind (lower (jcpentry), lower (entries2{jj}))))
-        jpchk2 = jpchk2 + 1; found = 1;
-        if (dbug > 2), fprintf ('  - %s OK\n', jcp{ii}); end %if
-      end %if
-    end % for
-    if (~found)
-      if (dbug > 2), fprintf ('  %s....jar missing\n', entries2{jj}); end %if
-      missing2(jj) = 1;
-    end %if
-  end % for
+  if (dbug > 1), fprintf ('\nPOI OOXML (.xlsx) <xbean/xmlbean> <poi-ooxml-schemas> <dom4j>:\n'); end %if
+  entries2 = {{'xbean', 'xmlbean'}, 'poi-ooxml-schemas', 'dom4j'}; 
   % Only update retval if all classes for basic POI have been found in javaclasspath
+  [jpchk2, missing2] = chk_jar_entries (jcp, entries2, dbug);
   if (jpchk1 >= numel (entries1) && jpchk2 >= numel (entries2)), retval = retval + 4; end %if
   if (dbug > 1)
     if (jpchk2 >= numel (entries2)) 
@@ -239,22 +231,8 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
 
   % Try Java & JExcelAPI
   if (dbug > 1), fprintf ('\nJExcelAPI (.xls (incl. BIFF5 read)) <jxl>:\n'); end %if
-  jpchk = 0; entries3 = {'jxl'}; missing3 = zeros (1, numel (entries3));
-  for jj=1:length (entries3)
-    found = 0;
-    for ii=1:length (jcp)
-      jcplst = strsplit (jcp{ii}, filesep);
-      jcpentry = jcplst {end};
-      if (~isempty (strfind (lower (jcpentry), lower (entries3{jj}))))
-        jpchk = jpchk + 1; found = 1;
-        if (dbug > 2), fprintf ('  - %s OK\n', jcp{ii}); end %if
-      end % if
-    end %for
-    if (~found) 
-      if (dbug > 2), fprintf ('  %s....jar missing\n', entries3{jj}); end %if 
-      missing3(jj) = 1; 
-    end %if
-  end %for
+  entries3 = {'jxl'}; missing3 = zeros (1, numel (entries3));
+  [jpchk, missing3] = chk_jar_entries (jcp, entries3, dbug);
   if (jpchk >= numel (entries3)), retval = retval + 8; end %if
   if (dbug > 1)
     if (jpchk >= numel (entries3))
@@ -266,22 +244,8 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
 
   % Try Java & OpenXLS
   if (dbug > 1), fprintf ('\nOpenXLS (.xls (BIFF8)) <OpenXLS>:\n'); end %if
-  jpchk = 0; entries4 = {'OpenXLS'}; missing4 = zeros (1, numel (entries4));
-  for jj=1:length (entries4)
-    found = 0;
-    for ii=1:length (jcp)
-      jcplst = strsplit (jcp{ii}, filesep);
-      jcpentry = jcplst {end};
-      if (~isempty (strfind (lower (jcpentry), lower (entries4{jj}))))
-        jpchk = jpchk + 1; found = 1;
-        if (dbug > 2), fprintf ('  - %s OK\n', jcp{ii}); end %if
-      end % if
-    end %for
-    if (~found) 
-      if (dbug > 2), fprintf ('  %s....jar missing\n', entries4{jj}); end %if 
-      missing4(jj) = 1; 
-    end %if
-  end %for
+  entries4 = {'OpenXLS'}; missing4 = zeros (1, numel (entries4));
+  [jpchk, missing4] = chk_jar_entries (jcp, entries4, dbug);
   if (jpchk >= numel (entries4)), retval = retval + 16; end %if
   if (dbug > 1)
     if (jpchk >= numel (entries4))
@@ -293,22 +257,8 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
 
   % Try Java & ODF toolkit
   if (dbug > 1), fprintf ('\nODF Toolkit (.ods) <odfdom> <xercesImpl>:\n'); end %if
-  jpchk = 0; entries5 = {'odfdom', 'xercesImpl'}; missing5 = zeros (1, numel (entries5));
-  for jj=1:length (entries5)
-    found = 0;
-    for ii=1:length (jcp)
-      jcplst = strsplit (jcp{ii}, filesep);
-      jcpentry = jcplst {end};
-      if (~isempty (strfind ( lower (jcpentry), lower (entries5{jj}))))
-        jpchk = jpchk + 1; found = 1;
-        if (dbug > 2), fprintf ('  - %s OK\n', jcp{ii}); end %if
-      end %if
-    end %for
-    if (~found) 
-      if (dbug > 2), fprintf ('  %s....jar missing\n', entries5{jj}); end %if
-      missing5(jj) = 1; 
-    end %if
-  end %for
+  entries5 = {'odfdom', 'xercesImpl'}; missing5 = zeros (1, numel (entries5));
+  [jpchk, missing5] = chk_jar_entries (jcp, entries5, dbug);
   if (jpchk >= numel (entries5))    % Apparently all requested classes present.
     % Only now we can check for proper odfdom version (only 0.7.5 & 0.8.6 work OK).
     % The odfdom team deemed it necessary to change the version call so we need this:
@@ -320,8 +270,9 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
       % Worked in 0.7.5
       odfvsn = javaMethod ('getApplicationVersion', 'org.odftoolkit.odfdom.Version');
     end %try_catch
-    if ~(strcmp (odfvsn, '0.7.5') || strcmp (odfvsn, '0.8.6') || strcmp (odfvsn, '0.8.7'))
-      warning ('  *** odfdom version (%s) is not supported - use v. 0.8.6 or 0.8.7.\n', odfvsn);
+    if ~(strcmp (odfvsn, '0.7.5') || strcmp (odfvsn, '0.8.6') || strcmp (odfvsn, '0.8.7')
+      || ~isempty (strfind (odfvsn, '0.8.8')))
+      warning ('  *** odfdom version (%s) is not supported - use v. 0.8.6 or newer\n', odfvsn);
     else  
       if (dbug > 1), fprintf ('  => ODFtoolkit (OTK) OK.\n'); end %if
       retval = retval + 32;
@@ -332,22 +283,8 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
 
   % Try Java & jOpenDocument
   if (dbug > 1), fprintf ('\njOpenDocument (.ods + experimental .sxc readonly) <jOpendocument>:\n'); end %if
-  jpchk = 0; entries6 = {'jOpenDocument'}; missing6 = zeros (1, numel (entries6));
-  for jj=1:length (entries6)
-    found = 0;
-    for ii=1:length (jcp)
-      jcplst = strsplit (jcp{ii}, filesep);
-      jcpentry = jcplst {end};
-       if (~isempty (strfind (lower (jcpentry), lower (entries6{jj}))))
-        jpchk = jpchk + 1; found = 1;
-        if (dbug > 2), fprintf ('  - %s OK\n', jcp{ii}); end %if
-      end %if
-    end %for
-    if (~found) 
-      if (dbug > 2), fprintf ('  %s....jar missing\n', entries6{jj}); end %if 
-      missing6(jj) = 1; 
-    end %if
-  end %for
+  entries6 = {'jOpenDocument'}; missing6 = zeros (1, numel (entries6));
+  [jpchk, missing6] = chk_jar_entries (jcp, entries6, dbug);
   if (jpchk >= numel (entries6)), retval = retval + 64; end %if
   if (dbug > 1)
     if (jpchk >= numel(entries6))
@@ -360,30 +297,8 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
   % Try Java & UNO
   if (dbug > 1), fprintf ('\nUNO/Java (.ods, .xls, .xlsx, .sxc) <OpenOffice.org>:\n'); end %if
   % entries0(1) = not a jar but a directory (<000_install_dir/program/>)
-  jpchk = 0; entries0 = {'program', 'unoil', 'jurt', 'juh', 'unoloader', 'ridl'};
-  missing0 = zeros (1, numel (entries0));
-  for jj=1:numel (entries0)
-    found = 0;
-    for ii=1:numel (jcp)
-      jcplst = strsplit (jcp{ii}, filesep);
-      jcpentry = jcplst {end};
-      if (~isempty (strfind (lower (jcpentry), lower (entries0{jj}))))
-        jpchk = jpchk + 1; found = 1;
-        if (dbug > 2), fprintf ('  - %s OK\n', jcp{ii}); end %if
-      end %if
-    end %for
-    if (~found)
-      if (dbug > 2)
-        if (jj == 1)
-          % Just a dir
-          fprintf ('  %s.... (directory) not found\n', entries0{jj}); 
-        else
-          fprintf ('  %s....jar missing\n', entries0{jj}); 
-        end %if
-      end %if 
-      missing0(jj) = 1; 
-    end %if
-  end %for
+  entries0 = {'program', 'unoil', 'jurt', 'juh', 'unoloader', 'ridl'};
+  [jpchk, missing0] = chk_jar_entries (jcp, entries0, dbug);
   if (jpchk >= numel (entries0)), retval = retval + 128; end %if
   if (dbug > 1)
     if (jpchk >= numel (entries0))
@@ -398,7 +313,7 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
 
   if (~ujars_complete && nargin > 0 && ~isempty (path_to_ooo))
     if (dbug), fprintf ('\nTrying to add missing UNO java class libs to javaclasspath...\n'); end %if
-    if (~ischar (path_to_jars)), error ('Path expected for arg # 1'); end %if
+    if (~ischar (path_to_jars)), printf ('Path expected for arg # 1\n'); return; end %if
     % Add missing jars to javaclasspath. First combine all entries
     targt = sum (missing0);
     if (missing0(1))
@@ -414,7 +329,10 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
           if (dbug > 2), fprintf ('FAILED\n'); end %if
         end %try_catch
       else
-        if (dbug > 2), error ('Suggested OpenOffice.org install directory: %s not found!\n', path_to_ooo); end %if
+        if (dbug > 2)
+          printf ('Suggested OpenOffice.org install directory: %s not found!\n', path_to_ooo); 
+          return
+        end %if
       end %if
     end %if
     % Rest of missing entries. Find where URE is located. Watch out because case of ./ure is unknown
@@ -492,7 +410,7 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
   jars_complete = isempty (find (missing, 1));
   if (dbug)
     if (jars_complete)
-      fprintf ('All interfaces already fully supported.\n\n');
+      fprintf ('All Java-based interfaces (save UNO) fully supported.\n\n');
     else
       fprintf ('Some class libs lacking yet...\n\n'); 
     end %if
@@ -501,29 +419,29 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
   if (~jars_complete && nargin > 0 && ~isempty (path_to_jars))
     % Add missing jars to javaclasspath. Assume they're all in the same place
     if (dbug), fprintf ('Trying to add missing java class libs to javaclasspath...\n'); end %if
-    if (~ischar (path_to_jars)), error ('Path expected for arg # 1'); end %if
+    if (~ischar (path_to_jars)), printf ('Path expected for arg # 1\n'); return; end %if
     % First combine all entries
     targt = sum (missing);
-    % Search tru list of missing entries
-    for ii=1:6   % Adapt in case of future new interfaces
+    % For each interface, search tru list of missing entries
+    for ii=1:6   % Adapt number in case of future new interfaces
       tmpe = eval ([ 'entries' char(ii + '0') ]);
       tmpm = eval ([ 'missing' char(ii + '0') ]);
       if (sum (tmpm))
         for jj=1:numel (tmpe)
           if (tmpm(jj))
-            file = dir ([path_to_jars filesep tmpe{jj} '*']);
-            if (isempty (file))
-              if (dbug > 2), fprintf ('  ? %s<...>.jar ?\n', tmpe{jj}); end %if
+            if (iscellstr (tmpe{jj}))
+              rtval = 0; kk = 1;
+              while (kk <= numel (tmpe{jj}) && ! rtval)
+                jtmpe = tmpe{jj}{kk};
+                rtval = add_jars_to_jcp (path_to_jars, jtmpe, dbug);
+                ++kk;
+              end %while
             else
-              if (dbug > 2), fprintf ('  Found %s, adding it to javaclasspath ... ', file(1).name); end %if
-              try
-                javaaddpath ([path_to_jars filesep file(1).name]);
-                targt = targt - 1;
-                tmpm(jj) = 0;
-                if (dbug > 2), fprintf ('OK\n'); end %if
-              catch
-                if (dbug > 2), fprintf ('FAILED\n'); end %if
-              end %try_catch
+              rtval = add_jars_to_jcp (path_to_jars, tmpe{jj}, dbug);
+            end %if
+            if (rtval)
+              targt = targt - rtval;
+              tmpm(jj) = 0;
             end %if
           end %if
         end %for
@@ -536,7 +454,7 @@ function  [ retval ]  = chk_spreadsheet_support (path_to_jars, dbug, path_to_ooo
       if (targt)
         fprintf ('Some other class libs still lacking...\n\n');
       else
-        fprintf ('All interfaces fully supported.now.\n\n');
+        fprintf ('All Java-based interfaces fully supported.now.\n\n');
       end %if
     end %f
   end %if
@@ -567,3 +485,27 @@ function [ ret_dir ] = get_dir_ (base_dir, req_dir)
   end %if
 
 end %function
+
+
+function [ retval ] = add_jars_to_jcp (path_to_jars, jarname, dbug)
+
+% Given a subdirectory path and a (sufficiently unique part of a) Java class
+% lib file (.jar), checks if it can find the file in the subdir and tries to
+% add it to the javaclasspath
+
+  retval = 0;
+  file = dir ([path_to_jars filesep jarname '*']);  %%% FIXME mult_jar
+  if (isempty (file))
+    if (dbug > 2), fprintf ('  ? %s<...>.jar ?\n', jarname); end %if
+  else
+    if (dbug > 2), fprintf ('  Found %s, adding it to javaclasspath ... ', file(1).name); end %if
+    try
+      javaaddpath ([path_to_jars filesep file(1).name]);
+      if (dbug > 2), fprintf ('OK\n'); end %if
+      retval = 1;
+    catch
+      if (dbug > 2), fprintf ('FAILED\n'); end %if
+    end %try_catch
+  end %if
+
+end %function
\ No newline at end of file
