@@ -95,13 +95,14 @@
 ## 2012-02-20 Fixed range parameter to be default empty string rather than empty numeral
 ## 2010-03-07 Updated texinfo help text
 ## 2012-06-08 Tabs replaced by double space
+## 2012-10-24 Style fixes
 
-function [ rstatus ] = odswrite (filename, data, wsh=1, crange='', reqintf=[])
+function [ rstatus ] = odswrite (filename, data, wsh=1, crange="", reqintf=[])
 
-  # Input validity checks
+  ## Input validity checks
   if (nargin < 2)
     usage ("Insufficient arguments - see 'help odswrite'");
-  elseif (~ischar (filename) || isempty (findstr ('.ods', tolower (filename))))
+  elseif (~ischar (filename) || isempty (findstr (".ods", tolower (filename))))
     error ("First argument must be a filename (incl. .ods suffix for OTK & JOD)");
   endif
 
@@ -110,7 +111,7 @@ function [ rstatus ] = odswrite (filename, data, wsh=1, crange='', reqintf=[])
   if (~isempty (ods)) 
     [ods, rstatus] = oct2ods (data, ods, wsh, crange);
 
-    # If rstatus was not OK, reset change indicator in ods pointer
+    ## If rstatus was not OK, reset change indicator in ods pointer
     if (~rstatus)
       ods.changed = rstatus;
       warning ("odswrite: data transfer errors, file not rewritten");
